package com.github.itzilly.sbt.features

import com.github.itzilly.sbt.SkyBlockTweaks
import com.github.itzilly.sbt.renderer.RenderFuncs
import net.minecraft.client.Minecraft
import net.minecraft.init.Blocks
import net.minecraft.util.BlockPos
import net.minecraftforge.client.event.RenderWorldLastEvent
import net.minecraftforge.common.config.Configuration
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent

enum class GemstoneColors(val color: Int) {
    RUBY(0xFF5555),     // Red
    JADE(0x00AA00),     // Lime
    AMBER(0xFFAA00),    // Orange
    SAPPHIRE(0x55FFFF), // Light Blue
    AMETHYST(0xAA00AA), // Purple
    TOPAZ(0xFFFF55),    // Yellow
    JASPER(0xFF55FF)    // Magenta
}

object GemstoneFinder {
    private var gemstoneList: ArrayList<GemstoneBlock> = ArrayList()
    var radius = SkyBlockTweaks.config.getCategory(Configuration.CATEGORY_GENERAL)?.get("gemstonesearchradius")?.int?: 16

    fun clear() {
        gemstoneList.clear()
    }

    fun find() {
        val playerPos = Minecraft.getMinecraft().thePlayer.position

        val minBounds = BlockPos(
            playerPos.x - radius,
            maxOf(playerPos.y - radius, 31),
            playerPos.z - radius
        )

        val maxBounds = BlockPos(
            playerPos.x + radius,
            minOf(playerPos.y + radius, 188),
            playerPos.z + radius
        )
        val world = Minecraft.getMinecraft().thePlayer.worldObj

        for (chunkX in minBounds.x / 16..maxBounds.x / 16) {
            for (chunkZ in minBounds.z / 16..maxBounds.z / 16) {
                val chunk = world.getChunkFromChunkCoords(chunkX, chunkZ)

                // Iterate through the chunk's blocks within the world bounds
                for (x in 0..15) {
                    for (y in minBounds.y..maxBounds.y) {
                        for (z in 0..15) {
                            val worldX = chunk.xPosition * 16 + x
                            val worldZ = chunk.zPosition * 16 + z

                            if (worldX in minBounds.x..maxBounds.x && worldZ in minBounds.z..maxBounds.z) {
                                val blockPos = BlockPos(worldX, y, worldZ)
                                val blockState = chunk.getBlockState(blockPos)
                                val block = blockState.block
                                if (block == Blocks.air) { continue }
                                val meta = block.getMetaFromState(blockState)

                                val gemstoneColor = when {
                                    block === Blocks.stained_glass || block === Blocks.stained_glass_pane -> {
                                        when (meta) {
                                            1 -> GemstoneColors.AMBER  // Orange
                                            2 -> GemstoneColors.JASPER  // Magenta
                                            3 -> GemstoneColors.SAPPHIRE  // Light Blue
                                            4 -> GemstoneColors.TOPAZ  // Yellow
                                            5 -> GemstoneColors.JADE  // Lime
                                            14 -> GemstoneColors.RUBY  // Red
                                            10 -> GemstoneColors.AMETHYST  // Purple

                                            else -> null
                                        }
                                    }
                                    else -> null
                                }
                                if (gemstoneColor != null && !gemstonesList(blockPos.x, blockPos.y, blockPos.z)) {
                                    gemstoneList.add(GemstoneBlock(blockPos.x, blockPos.y, blockPos.z, gemstoneColor))
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    @SubscribeEvent
    fun renderBlockOverlay(event: RenderWorldLastEvent) {
        val player = Minecraft.getMinecraft().thePlayer

        for (gemstone: GemstoneBlock in gemstoneList) {
            val pos = BlockPos(gemstone.x, gemstone.y, gemstone.z)
            RenderFuncs.drawBlockOutline(player, pos, event.partialTicks, gemstone.color.color)
        }
    }

    data class GemstoneBlock(
        val x: Int,
        val y: Int,
        val z: Int,
        val color: GemstoneColors
    )

    fun gemstonesList(x: Int, y: Int, z: Int): Boolean {
        return gemstoneList.any { it.x == x && it.y == y && it.z == z }
    }
}